@@ -47,6 +47,7 @@ use PKP\security\authorization\PublicationWritePolicy;
 use PKP\security\authorization\StageRolePolicy;
 use PKP\security\authorization\SubmissionAccessPolicy;
 use PKP\security\authorization\UserRolesRequiredPolicy;
+use PKP\security\authorization\internal\SubmissionCompletePolicy;
 use PKP\security\Role;
 use PKP\security\Validation;
 use PKP\services\PKPSchemaService;
@@ -285,6 +286,7 @@ class PKPSubmissionHandler extends APIHandler
         }
 
         if ($routeName === 'addDecision') {
+            $this->addPolicy(new SubmissionCompletePolicy($request, $args));
             $this->addPolicy(new DecisionWritePolicy($request, $args, (int) $request->getUserVar('decision'), $request->getUser()));
         }
 
@@ -333,7 +335,7 @@ class PKPSubmissionHandler extends APIHandler
         $genres = $genreDao->getByContextId($context->getId())->toArray();
 
         return $response->withJson([
-            'itemsMax' => $collector->limit(null)->offset(null)->getCount(),
+            'itemsMax' => $collector->getCount(),
             'items' => Repo::submission()->getSchemaMap()->summarizeMany($submissions, $userGroups, $genres)->values(),
         ], 200);
     }
@@ -581,23 +583,28 @@ class PKPSubmissionHandler extends APIHandler
         $request = $this->getRequest();
         $submission = $this->getAuthorizedContextObject(Application::ASSOC_TYPE_SUBMISSION);
 
-        $params = $this->convertStringsToSchema(PKPSchemaService::SCHEMA_SUBMISSION, $slimRequest->getParsedBody());
+        $parsedBody = $slimRequest->getParsedBody();
 
-        $readOnlyErrors = $this->getWriteDisabledErrors(PKPSchemaService::SCHEMA_SUBMISSION, $params);
-        if (!empty($readOnlyErrors)) {
-            return $response->withStatus(400)->withJson($readOnlyErrors);
+        if (isset($parsedBody)){
+            $params = $this->convertStringsToSchema(PKPSchemaService::SCHEMA_SUBMISSION, $parsedBody);
+    
+            $readOnlyErrors = $this->getWriteDisabledErrors(PKPSchemaService::SCHEMA_SUBMISSION, $params);
+            if (!empty($readOnlyErrors)) {
+                return $response->withStatus(400)->withJson($readOnlyErrors);
+            }
+    
+            $params['id'] = $submission->getId();
+            $params['contextId'] = $request->getContext()->getId();
+    
+            $errors = Repo::submission()->validate($submission, $params, $request->getContext());
+    
+            if (!empty($errors)) {
+                return $response->withStatus(400)->withJson($errors);
+            }
+    
+            Repo::submission()->edit($submission, $params);
         }
 
-        $params['id'] = $submission->getId();
-        $params['contextId'] = $request->getContext()->getId();
-
-        $errors = Repo::submission()->validate($submission, $params, $request->getContext());
-
-        if (!empty($errors)) {
-            return $response->withStatus(400)->withJson($errors);
-        }
-
-        Repo::submission()->edit($submission, $params);
 
         $submission = Repo::submission()->get($submission->getId());
 
@@ -882,7 +889,7 @@ class PKPSubmissionHandler extends APIHandler
         $genres = $genreDao->getByContextId($submission->getData('contextId'))->toArray();
 
         return $response->withJson([
-            'itemsMax' => $collector->limit(null)->offset(null)->getCount(),
+            'itemsMax' => $collector->getCount(),
             'items' => Repo::publication()->getSchemaMap($submission, $userGroups, $genres)->summarizeMany($publications, $anonymize)->values(),
         ], 200);
     }
@@ -955,7 +962,6 @@ class PKPSubmissionHandler extends APIHandler
             return $response->withStatus(400)->withJson($errors);
         }
 
-        $params = (new \PKP\submission\Sanitizer())->sanitize($params, ['title']);
         $publication = Repo::publication()->newDataObject($params);
         $newId = Repo::publication()->add($publication);
         $publication = Repo::publication()->get($newId);
@@ -1114,7 +1120,6 @@ class PKPSubmissionHandler extends APIHandler
             return $response->withStatus(400)->withJson($errors);
         }
 
-        $params = (new \PKP\submission\Sanitizer())->sanitize($params, ['title', 'subtitle']);
         Repo::publication()->edit($publication, $params);
 
         $publication = Repo::publication()->get($publication->getId());
@@ -1349,7 +1354,7 @@ class PKPSubmissionHandler extends APIHandler
         $authors = $collector->getMany();
 
         return $response->withJson([
-            'itemsMax' => $collector->limit(null)->offset(null)->getCount(),
+            'itemsMax' => $collector->getCount(),
             'items' => Repo::author()->getSchemaMap()->summarizeMany($authors)->values(),
         ], 200);
     }
