@@ -29,7 +29,6 @@ use PKP\db\DAORegistry;
 use PKP\facades\Locale;
 use PKP\filter\FilterGroup;
 use PKP\submission\SubmissionLanguageDAO;
-use PKP\submission\SubmissionSubjectDAO;
 
 class MonographONIX30XmlFilter extends \PKP\plugins\importexport\native\filter\NativeExportFilter
 {
@@ -254,22 +253,21 @@ class MonographONIX30XmlFilter extends \PKP\plugins\importexport\native\filter\N
 
         /* --- Collection information, first for series and then for product --- */
 
-        $seriesCollectionNode = $doc->createElementNS($deployment->getNamespace(), 'Collection');
-        $seriesCollectionNode->appendChild($this->_buildTextNode($doc, 'CollectionType', '10')); // publisher series.
-        $descDetailNode->appendChild($seriesCollectionNode);
-
-        $seriesTitleDetailNode = $doc->createElementNS($deployment->getNamespace(), 'TitleDetail');
-        $seriesTitleDetailNode->appendChild($this->_buildTextNode($doc, 'TitleType', '01'));
-        $seriesCollectionNode->appendChild($seriesTitleDetailNode);
-
-        $titleElementNode = $doc->createElementNS($deployment->getNamespace(), 'TitleElement');
-        $titleElementNode->appendChild($this->_buildTextNode($doc, 'TitleElementLevel', '02')); // Collection level title
-        $seriesTitleDetailNode->appendChild($titleElementNode);
-
         /* --- Series information, if this monograph is part of one. --- */
         $seriesId = $submission->getCurrentPublication()->getData('seriesId');
         $series = $seriesId ? Repo::section()->get($seriesId) : null;
         if ($series != null) {
+            $seriesCollectionNode = $doc->createElementNS($deployment->getNamespace(), 'Collection');
+            $seriesCollectionNode->appendChild($this->_buildTextNode($doc, 'CollectionType', '10')); // publisher series.
+
+            $seriesTitleDetailNode = $doc->createElementNS($deployment->getNamespace(), 'TitleDetail');
+            $seriesTitleDetailNode->appendChild($this->_buildTextNode($doc, 'TitleType', '01'));
+            $seriesCollectionNode->appendChild($seriesTitleDetailNode);
+
+            $titleElementNode = $doc->createElementNS($deployment->getNamespace(), 'TitleElement');
+            $titleElementNode->appendChild($this->_buildTextNode($doc, 'TitleElementLevel', '02')); // Collection level title
+            $seriesTitleDetailNode->appendChild($titleElementNode);
+
             if ($submission->getCurrentPublication()->getData('seriesPosition')) {
                 $titleElementNode->appendChild($this->_buildTextNode($doc, 'PartNumber', $submission->getCurrentPublication()->getData('seriesPosition')));
             }
@@ -279,6 +277,8 @@ class MonographONIX30XmlFilter extends \PKP\plugins\importexport\native\filter\N
             } else {
                 if ($series->getLocalizedPrefix() != '') {
                     $titleElementNode->appendChild($this->_buildTextNode($doc, 'TitlePrefix', $series->getLocalizedPrefix()));
+                } else {
+                    $titleElementNode->appendChild($doc->createElementNS($deployment->getNamespace(), 'NoPrefix'));
                 }
 
                 $titleElementNode->appendChild($this->_buildTextNode($doc, 'TitleWithoutPrefix', $series->getLocalizedTitle(false)));
@@ -287,7 +287,10 @@ class MonographONIX30XmlFilter extends \PKP\plugins\importexport\native\filter\N
             if ($series->getLocalizedSubtitle() != '') {
                 $titleElementNode->appendChild($this->_buildTextNode($doc, 'Subtitle', $series->getLocalizedSubtitle()));
             }
+        } else {
+            $seriesCollectionNode = $doc->createElementNS($deployment->getNamespace(), 'NoCollection');
         }
+        $descDetailNode->appendChild($seriesCollectionNode);
 
         /* --- and now product level info --- */
 
@@ -306,7 +309,10 @@ class MonographONIX30XmlFilter extends \PKP\plugins\importexport\native\filter\N
         } else {
             if ($publication->getLocalizedData('prefix')) {
                 $titleElementNode->appendChild($this->_buildTextNode($doc, 'TitlePrefix', $publication->getLocalizedData('prefix')));
+            } else {
+                $titleElementNode->appendChild($doc->createElementNS($deployment->getNamespace(), 'NoPrefix'));
             }
+
             $titleElementNode->appendChild($this->_buildTextNode($doc, 'TitleWithoutPrefix', strip_tags($publication->getLocalizedData('title'))));
         }
 
@@ -324,9 +330,10 @@ class MonographONIX30XmlFilter extends \PKP\plugins\importexport\native\filter\N
 
             $userGroup = Repo::userGroup()->get($author->getUserGroupId());
 
-            $userGroupOnixMap = ['AU' => 'A01', 'VE' => 'B01', 'CA' => 'A01', 'Trans' => 'B06', 'PE' => 'B21']; // From List17, ContributorRole types.
+            $userGroupOnixMap = ['default.groups.name.author' => 'A01', 'default.groups.name.volumeEditor' => 'B01', 'default.groups.name.chapterAuthor' => 'A01', 'default.groups.name.translator' => 'B06', 'default.groups.name.editor' => 'B21']; // From List17, ContributorRole types.
 
-            $role = array_key_exists($userGroup->getLocalizedAbbrev(), $userGroupOnixMap) ? $userGroupOnixMap[$userGroup->getLocalizedAbbrev()] : 'Z99'; // Z99 - unknown contributor type.
+            $nameKey = $userGroup->getData('nameLocaleKey');
+            $role = array_key_exists($nameKey, $userGroupOnixMap) ? $userGroupOnixMap[$nameKey] : 'Z99'; // Z99 - unknown contributor type.
 
             $contributorNode->appendChild($this->_buildTextNode($doc, 'ContributorRole', $role));
             $contributorNode->appendChild($this->_buildTextNode($doc, 'PersonName', $author->getFullName(false)));
@@ -359,7 +366,7 @@ class MonographONIX30XmlFilter extends \PKP\plugins\importexport\native\filter\N
         }
 
         if (sizeof($authors) == 0) { // this will probably never happen, but include the possibility.
-            $descDetailNode->appendChild($this->_buildTextNode($doc, 'NoContributor', '')); // empty state of fact.
+            $descDetailNode->appendChild($doc->createElementNS($deployment->getNamespace(), 'NoContributor')); // empty state of fact.
         }
 
         /* --- Add Language elements --- */
@@ -405,28 +412,28 @@ class MonographONIX30XmlFilter extends \PKP\plugins\importexport\native\filter\N
             unset($extentNode);
         }
 
-
         /* --- Add Subject elements --- */
 
-        $subjectNode = $doc->createElementNS($deployment->getNamespace(), 'Subject');
-        $mainSubjectNode = $doc->createElementNS($deployment->getNamespace(), 'MainSubject'); // Always empty as per 3.0 spec.
-        $subjectNode->appendChild($mainSubjectNode);
-        $subjectNode->appendChild($this->_buildTextNode($doc, 'SubjectSchemeIdentifier', '12')); // 12 is BIC subject category code list.
-        $subjectNode->appendChild($this->_buildTextNode($doc, 'SubjectSchemeVersion', '2')); // Version 2 of ^^
+        if ($publication->getData('subjects')) {
+            $subjectNode = $doc->createElementNS($deployment->getNamespace(), 'Subject');
+            $mainSubjectNode = $doc->createElementNS($deployment->getNamespace(), 'MainSubject'); // Always empty as per 3.0 spec.
 
-        $submissionSubjectDao = DAORegistry::getDAO('SubmissionSubjectDAO'); /** @var SubmissionSubjectDAO $submissionSubjectDao */
-        $allSubjects = $submissionSubjectDao->getSubjects($publication->getId(), array_keys(Locale::getSupportedFormLocales()));
-        $uniqueSubjects = [];
-        foreach ($allSubjects as $locale => $subjects) {
-            $uniqueSubjects = array_merge($uniqueSubjects, $subjects);
+            $subjectNode->appendChild($mainSubjectNode);
+            $subjectNode->appendChild($this->_buildTextNode($doc, 'SubjectSchemeIdentifier', '12')); // 12 is BIC subject category code list.
+            $subjectNode->appendChild($this->_buildTextNode($doc, 'SubjectSchemeVersion', '2')); // Version 2 of ^^
+
+            $allSubjects = ($publication->getData('subjects')[$publication->getData('locale')]);
+            $subjectNode->appendChild($this->_buildTextNode($doc, 'SubjectCode', trim(join(', ', $allSubjects))));
+            $descDetailNode->appendChild($subjectNode);
         }
 
-        if (sizeof($uniqueSubjects) > 0) {
-            $subjectNode->appendChild($this->_buildTextNode($doc, 'SubjectCode', trim(join(', ', $uniqueSubjects))));
+        if ($publication->getData('keywords')) {
+            $allKeywords = ($publication->getData('keywords')[$publication->getData('locale')]);
+            $keywordNode = $doc->createElementNS($deployment->getNamespace(), 'Subject');
+            $keywordNode->appendChild($this->_buildTextNode($doc, 'SubjectSchemeIdentifier', '20')); // Keywords
+            $keywordNode->appendChild($this->_buildTextNode($doc, 'SubjectHeadingText', trim(join(', ', $allKeywords))));
+            $descDetailNode->appendChild($keywordNode);
         }
-
-        $descDetailNode->appendChild($subjectNode);
-
         /* --- Add Audience elements --- */
 
         if ($submission->getData('audience')) {
@@ -480,6 +487,17 @@ class MonographONIX30XmlFilter extends \PKP\plugins\importexport\native\filter\N
         $textContentNode->appendChild($this->_buildTextNode($doc, 'ContentAudience', '00')); // Any audience
         $textContentNode->appendChild($this->_buildTextNode($doc, 'Text', $abstract)); // Any audience
 
+        $supportingResourceNode = $doc->createElementNS($deployment->getNamespace(), 'SupportingResource');
+        $collateralDetailNode->appendChild($supportingResourceNode);
+        $supportingResourceNode->appendChild($this->_buildTextNode($doc, 'ResourceContentType', '01')); // Front cover
+        $supportingResourceNode->appendChild($this->_buildTextNode($doc, 'ContentAudience', '00')); // Any audience
+        $supportingResourceNode->appendChild($this->_buildTextNode($doc, 'ResourceMode', '03')); // A still image
+
+        $resourceVersionNode = $doc->createElementNS($deployment->getNamespace(), 'ResourceVersion');
+        $supportingResourceNode->appendChild($resourceVersionNode);
+        $resourceVersionNode->appendChild($this->_buildTextNode($doc, 'ResourceForm', '01')); // Linkable resource
+        $resourceVersionNode->appendChild($this->_buildTextNode($doc, 'ResourceLink', $publication->getLocalizedCoverImageUrl($context->getId(), $publication->getData('locale'))));
+
         /* --- Publishing Detail --- */
 
         $publishingDetailNode = $doc->createElementNS($deployment->getNamespace(), 'PublishingDetail');
@@ -507,6 +525,19 @@ class MonographONIX30XmlFilter extends \PKP\plugins\importexport\native\filter\N
         $websiteNode->appendChild($this->_buildTextNode($doc, 'WebsiteRole', '18')); // 18 -> Publisher's B2C website
         $websiteNode->appendChild($this->_buildTextNode($doc, 'WebsiteLink', $request->url($context->getPath())));
 
+        $websiteNode = $doc->createElementNS($deployment->getNamespace(), 'Website');
+        $publisherNode->appendChild($websiteNode);
+
+        $websiteNode->appendChild($this->_buildTextNode($doc, 'WebsiteRole', '29')); // 29 -> Web page for full content
+
+        $submissionBestId = $publication->getData('submissionId');
+
+        if ($publication->getData('urlPath') != '') {
+            $submissionBestId = $publication->getData('urlPath');
+        }
+
+        $websiteNode->appendChild($this->_buildTextNode($doc, 'WebsiteLink', $request->url($context->getPath(), 'catalog', 'book', $submissionBestId)));
+
         /* --- Publishing Dates --- */
 
         $publicationDates = $publicationFormat->getPublicationDates();
@@ -531,34 +562,48 @@ class MonographONIX30XmlFilter extends \PKP\plugins\importexport\native\filter\N
         $allSalesRights = $publicationFormat->getSalesRights();
         $salesRightsROW = null;
         while ($salesRights = $allSalesRights->next()) {
-            if (!$salesRights->getROWSetting()) {
-                $salesRightsNode = $doc->createElementNS($deployment->getNamespace(), 'SalesRights');
-                $salesRightsNode->appendChild($this->_buildTextNode($doc, 'SalesRightsType', $salesRights->getType()));
+            $salesRightsNode = $doc->createElementNS($deployment->getNamespace(), 'SalesRights');
+            $publishingDetailNode->appendChild($salesRightsNode);
+            $salesRightsNode->appendChild($this->_buildTextNode($doc, 'SalesRightsType', $salesRights->getType()));
 
-                // now do territories and countries.
-                $territoryNode = $doc->createElementNS($deployment->getNamespace(), 'Territory');
+            // now do territories and countries.
+            $territoryNode = $doc->createElementNS($deployment->getNamespace(), 'Territory');
+            $salesRightsNode->appendChild($territoryNode);
 
-                if (sizeof($salesRights->getRegionsIncluded()) > 0 && sizeof($salesRights->getCountriesExcluded()) > 0) {
-                    $territoryNode->appendChild($this->_buildTextNode($doc, 'RegionsIncluded', trim(join(' ', $salesRights->getRegionsIncluded()))));
-                    $territoryNode->appendChild($this->_buildTextNode($doc, 'CountriesExcluded', trim(join(' ', $salesRights->getCountriesExcluded()))));
-                } elseif (sizeof($salesRights->getCountriesIncluded()) > 0) {
-                    $territoryNode->appendChild($this->_buildTextNode($doc, 'CountriesIncluded', trim(join(' ', $salesRights->getCountriesIncluded()))));
-                }
+            $salesCountriesIncluded = sizeof($salesRights->getCountriesIncluded()) > 0;
+            $salesRegionsIncluded = sizeof($salesRights->getRegionsIncluded()) > 0;
+            $salesCountriesExcluded = sizeof($salesRights->getCountriesExcluded()) > 0;
+            $salesRegionsExcluded = sizeof($salesRights->getRegionsExcluded()) > 0;
 
-                if (sizeof($salesRights->getRegionsExcluded()) > 0) {
+            if ($salesRights->getROWSetting()) {
+                $territoryNode->appendChild($this->_buildTextNode($doc, 'RegionsIncluded', 'WORLD'));
+                $salesRightsROW = $salesRights;
+            } elseif ($salesCountriesIncluded) {
+                $territoryNode->appendChild($this->_buildTextNode($doc, 'CountriesIncluded', trim(join(' ', $salesRights->getCountriesIncluded()))));
+                if (!in_array('WORLD', $salesRights->getRegionsIncluded())) {
+                    if ($salesRegionsIncluded) {
+                        $territoryNode->appendChild($this->_buildTextNode($doc, 'RegionsIncluded', trim(join(' ', $salesRights->getRegionsIncluded()))));
+                    }
+                    if ($salesRegionsExcluded) {
+                        $territoryNode->appendChild($this->_buildTextNode($doc, 'RegionsExcluded', trim(join(' ', $salesRights->getRegionsExcluded()))));
+                    }
+                } elseif ($salesRegionsExcluded) {
                     $territoryNode->appendChild($this->_buildTextNode($doc, 'RegionsExcluded', trim(join(' ', $salesRights->getRegionsExcluded()))));
                 }
-
-                // Include territory and sales rights if the territory isn't empty
-                if ($territoryNode->firstElementChild) {
-                    $salesRightsNode->appendChild($territoryNode);
-                    $publishingDetailNode->appendChild($salesRightsNode);
-                } else {
-                    $deployment->addWarning(Application::ASSOC_TYPE_MONOGRAPH, $deployment->getSubmission()->getId(), __('plugins.importexport.common.error.salesRightRequiresTerritory'));
+            } elseif ($salesRegionsIncluded) {
+                $territoryNode->appendChild($this->_buildTextNode($doc, 'RegionsIncluded', trim(join(' ', $salesRights->getRegionsIncluded()))));
+                if (in_array('WORLD', $salesRights->getRegionsIncluded())) {
+                    if ($salesCountriesExcluded) {
+                        $territoryNode->appendChild($this->_buildTextNode($doc, 'CountriesExcluded', trim(join(' ', $salesRights->getCountriesExcluded()))));
+                    }
+                    if ($salesRegionsExcluded) {
+                        $territoryNode->appendChild($this->_buildTextNode($doc, 'RegionsExcluded', trim(join(' ', $salesRights->getRegionsExcluded()))));
+                    }
                 }
-            } else { // found the SalesRights object that is assigned 'rest of world'.
-                $salesRightsROW = $salesRights; // stash this for later since it always goes last.
             }
+
+            unset($territoryNode);
+            unset($salesRightsNode);
             unset($salesRights);
         }
         if ($salesRightsROW != null) {
@@ -579,20 +624,33 @@ class MonographONIX30XmlFilter extends \PKP\plugins\importexport\native\filter\N
 
             $territoryNode = $doc->createElementNS($deployment->getNamespace(), 'Territory');
 
-            if (sizeof($market->getCountriesIncluded()) > 0) {
+            $marketCountriesIncluded = sizeof($market->getCountriesIncluded()) > 0;
+            $marketRegionsIncluded = sizeof($market->getRegionsIncluded()) > 0;
+            $marketCountriesExcluded = sizeof($market->getCountriesExcluded()) > 0;
+            $marketRegionsExcluded = sizeof($market->getRegionsExcluded()) > 0;
+
+            if ($marketCountriesIncluded) {
                 $territoryNode->appendChild($this->_buildTextNode($doc, 'CountriesIncluded', trim(join(' ', $market->getCountriesIncluded()))));
-            }
-
-            if (sizeof($market->getRegionsIncluded()) > 0) {
+                if (!in_array('WORLD', $market->getRegionsIncluded())) {
+                    if ($marketRegionsIncluded) {
+                        $territoryNode->appendChild($this->_buildTextNode($doc, 'RegionsIncluded', trim(join(' ', $market->getRegionsIncluded()))));
+                    }
+                    if ($marketRegionsExcluded) {
+                        $territoryNode->appendChild($this->_buildTextNode($doc, 'RegionsExcluded', trim(join(' ', $market->getRegionsExcluded()))));
+                    }
+                } elseif ($marketRegionsExcluded) {
+                    $territoryNode->appendChild($this->_buildTextNode($doc, 'RegionsExcluded', trim(join(' ', $market->getRegionsExcluded()))));
+                }
+            } elseif ($marketRegionsIncluded) {
                 $territoryNode->appendChild($this->_buildTextNode($doc, 'RegionsIncluded', trim(join(' ', $market->getRegionsIncluded()))));
-            }
-
-            if (sizeof($market->getCountriesExcluded()) > 0) {
-                $territoryNode->appendChild($this->_buildTextNode($doc, 'CountriesExcluded', trim(join(' ', $market->getCountriesExcluded()))));
-            }
-
-            if (sizeof($market->getRegionsExcluded()) > 0) {
-                $territoryNode->appendChild($this->_buildTextNode($doc, 'RegionsExcluded', trim(join(' ', $market->getRegionsExcluded()))));
+                if (in_array('WORLD', $market->getRegionsIncluded())) {
+                    if ($marketCountriesExcluded) {
+                        $territoryNode->appendChild($this->_buildTextNode($doc, 'CountriesExcluded', trim(join(' ', $market->getCountriesExcluded()))));
+                    }
+                    if ($marketRegionsExcluded) {
+                        $territoryNode->appendChild($this->_buildTextNode($doc, 'RegionsExcluded', trim(join(' ', $market->getRegionsExcluded()))));
+                    }
+                }
             }
 
             // Include territory if it's not empty
@@ -669,6 +727,13 @@ class MonographONIX30XmlFilter extends \PKP\plugins\importexport\native\filter\N
 
                     unset($supplierWebsiteNode);
                 }
+
+                $supplierWebsiteNode = $doc->createElementNS($deployment->getNamespace(), 'Website');
+                $supplierNode->appendChild($supplierWebsiteNode);
+
+                $supplierWebsiteNode->appendChild($this->_buildTextNode($doc, 'WebsiteRole', '29')); // 29 -> Web page for full content
+                $supplierWebsiteNode->appendChild($this->_buildTextNode($doc, 'WebsiteLink', $request->url($context->getPath(), 'catalog', 'book', $submissionBestId)));
+
                 unset($supplierNode);
             } else { // No suppliers specified, use the Press settings instead.
                 $supplierNode = $doc->createElementNS($deployment->getNamespace(), 'Supplier');
@@ -710,8 +775,14 @@ class MonographONIX30XmlFilter extends \PKP\plugins\importexport\native\filter\N
             $priceNode = $doc->createElementNS($deployment->getNamespace(), 'Price');
             $supplyDetailNode->appendChild($priceNode);
 
+            $excludeTaxNode = false;
+
             if ($market->getPriceTypeCode() != '') {
                 $priceNode->appendChild($this->_buildTextNode($doc, 'PriceType', $market->getPriceTypeCode()));
+                $priceTypeTaxEx = ['02', '04', '07', '09', '12', '14', '17', '22', '24', '27', '34', '42']; // Price type codes that include tax
+                if (in_array($market->getPriceTypeCode(), $priceTypeTaxEx)) {
+                    $excludeTaxNode = true;
+                }
             }
 
             if ($market->getDiscount() != '') {
@@ -723,7 +794,7 @@ class MonographONIX30XmlFilter extends \PKP\plugins\importexport\native\filter\N
 
             $priceNode->appendChild($this->_buildTextNode($doc, 'PriceAmount', $market->getPrice()));
 
-            if ($market->getTaxTypeCode() != '' || $market->getTaxRateCode() != '') {
+            if (!$excludeTaxNode && ($market->getTaxTypeCode() != '' || $market->getTaxRateCode() != '')) {
                 $taxNode = $doc->createElementNS($deployment->getNamespace(), 'Tax');
                 $priceNode->appendChild($taxNode);
 
@@ -732,7 +803,11 @@ class MonographONIX30XmlFilter extends \PKP\plugins\importexport\native\filter\N
                 }
                 if ($market->getTaxRateCode()) {
                     $taxNode->appendChild($this->_buildTextNode($doc, 'TaxRateCode', $market->getTaxRateCode()));
+                    if ($market->getTaxRateCode() == 'Z') {
+                        $taxNode->appendChild($this->_buildTextNode($doc, 'TaxRatePercent', '0')); // Zero-rated tax rate type
+                    }
                 }
+                $taxNode->appendChild($this->_buildTextNode($doc, 'TaxableAmount', $market->getPrice())); // Taxable amount defaults to full price
                 unset($taxNode);
             }
 
