@@ -154,7 +154,7 @@ class Dc11SchemaPublicationFormatAdapter extends MetadataDataObjectAdapter
         // Identifier: URL
         $request = Application::get()->getRequest();
         if ($monograph instanceof Submission) {
-            $dc11Description->addStatement('dc:identifier', $request->url($press->getPath(), 'catalog', 'book', [$monograph->getId()]));
+            $dc11Description->addStatement('dc:identifier', $request->url($press->getPath(), 'catalog', 'book', [$publication->getData('urlPath') ?? $monograph->getId()]));
         }
 
         // Public identifiers (e.g. DOI, URN)
@@ -164,6 +164,11 @@ class Dc11SchemaPublicationFormatAdapter extends MetadataDataObjectAdapter
             if ($pubId) {
                 $dc11Description->addStatement('dc:identifier', $pubId);
             }
+            $publicationPubId = $plugin->getPubId($publication);
+            if ($publicationPubId) {
+                $dc11Description->addStatement('dc:relation', $publicationPubId->getData('doi'));
+            }
+
         }
         $context = $request->getContext();
         if (!$context) {
@@ -176,6 +181,10 @@ class Dc11SchemaPublicationFormatAdapter extends MetadataDataObjectAdapter
             if ($doi) {
                 $dc11Description->addStatement('dc:identifier', $doi);
             }
+            $publicationDoi = $publication->getData('doiObject');
+            if ($publicationDoi) {
+                $dc11Description->addStatement('dc:relation', $publicationDoi->getData('doi'));
+            }
         }
 
         // Identifier: others
@@ -212,7 +221,7 @@ class Dc11SchemaPublicationFormatAdapter extends MetadataDataObjectAdapter
         foreach ($pubFormatFiles as $file) {
             {
                 if ($file->getData('assocId') == $publicationFormat->getData('id')) {
-                    $relation = $request->url($press->getData('urlPath'), 'catalog', 'view', [$monograph->getId(), $publicationFormat->getId(), $file->getId()]);
+                    $relation = $request->url($press->getData('urlPath'), 'catalog', 'view', [$publication->getData('urlPath') ?? $monograph->getId(), $publicationFormat->getId(), $file->getId()]);
                     $dc11Description->addStatement('dc:relation', $relation);
                 }
             }
